/*
  Copyright 2010, 2010 STARWest

  See the file STARWest.txt for details about the STARWest research group

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Facette Carrée H1F1 pour des héliostat a facette (H1)

// H1F1.cpp*
#include "shape.h"
//#include "mc.h" inutile?

// H1F1 Declarations
class H1F1: public Shape {
public:
	// H1F1 Public Methods
	H1F1(const Transform &o2w, bool ro, float cote);
	BBox ObjectBound() const;
  	bool Intersect(const Ray &ray, float *tHit,
  	               DifferentialGeometry *dg) const;
  	bool IntersectP(const Ray &ray) const;
	float Area() const;
	Point Sample(float u1, float u2, Normal *ns) const {
		Point p = Point((u1-0.5)*cote,(u2-0.5)*cote,0);
		*ns = Normalize(ObjectToWorld(Normal(0.,0.,1.)));
		if (reverseOrientation) *ns *= -1.f;
		return ObjectToWorld(p);
	}


private:
	// H1F1 Private Data
	float cote;
};
// H1F1 Method Definitions
H1F1::H1F1(const Transform &o2w, bool ro, float c)
	: Shape(o2w, ro) {
        cote = c;
}
BBox H1F1::ObjectBound() const {
  return BBox(Point(-cote/2,-cote/2,0),
	      Point(cote/2,cote/2,0));
}

bool H1F1::Intersect(const Ray &r, float *tHit,
		DifferentialGeometry *dg) const {
    
	// Transform _Ray_ to object space
	Ray ray;
	WorldToObject(r, &ray);
	// Intersection avec le carre
	float xH,yH,t;
	
	// Si le rayon parallele est au plan du carre
	if(fabsf(ray.d.z) < 1e-7) return false;

	t=-ray.o.z/ray.d.z;
	xH=ray.o.x+t*ray.d.x;
	yH=ray.o.y+t*ray.d.y;
	if( Clamp(xH,-cote/2.,cote/2.) != xH
	    || Clamp(yH,-cote/2.,cote/2.) != yH )
	  return false;
	  
	
	// Compute intersection distance along ray
	if (t > ray.maxt || t < ray.mint)
		return false;
	float thit = t;
	
	// Point d'intersection sur le carre
	Point phit=ray(thit);



	// Find parametric representation of cube hit
	Vector dpdu(cote,0,0);
	Vector dpdv(0,cote,0);
	float u = phit.x/cote+0.5;
	float v = phit.y/cote+0.5;
	Vector dndu(0,0,0), dndv(0,0,0);
	// Initialize _DifferentialGeometry_ from parametric information
	*dg = DifferentialGeometry(ObjectToWorld(phit),
	                           ObjectToWorld(dpdu),
				   ObjectToWorld(dpdv),
	                           ObjectToWorld(dndu),
				   ObjectToWorld(dndv),
	                           u, v, this);

	// Update tHit 
	*tHit = t;
	return true;
}

bool H1F1::IntersectP(const Ray &r) const {

	// Transform _Ray_ to object space
	Ray ray;
	WorldToObject(r, &ray);
	// Intersection avec le carre
	float xH,yH,t;
	
	// Si le rayon parallele est au plan du carre
	if(ray.d.z==0)return false;

	t=-ray.o.z/ray.d.z;
	xH=ray.o.x+t*ray.d.x;
	yH=ray.o.y+t*ray.d.y;
	if( Clamp(xH,-cote/2.,cote/2.) != xH 
	    || Clamp(yH,-cote/2.,cote/2.) != yH )
	  return false;

	  
	
	// Compute intersection distance along ray
	if (t > ray.maxt || t < ray.mint)
	  return false;       

	return true;
}

float H1F1::Area() const {
	return cote*cote;
}
extern "C" DLLEXPORT
	Shape *CreateShape(const Transform &o2w,
					   bool reverseOrientation,
					   const ParamSet &params) {
	float cote = params.FindOneFloat("cote", 1.f);

	return new H1F1(o2w, reverseOrientation, cote);
}
